/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * This header provides macros for Atmel Flexcom DT bindings.
 *
 * Copyright (C) 2015 Cyrille Pitchen <cyrille.pitchen@atmel.com>
 */

#ifndef __DT_BINDINGS_ATMEL_FLEXCOM_H__
#define __DT_BINDINGS_ATMEL_FLEXCOM_H__

#define ATMEL_FLEXCOM_MODE_USART	1
#define ATMEL_FLEXCOM_MODE_SPI		2
#define ATMEL_FLEXCOM_MODE_TWI		3

#endif /* __DT_BINDINGS_ATMEL_FLEXCOM_H__ */
