/* SPDX-License-Identifier: (GPL-2.0+ OR BSD-3-Clause) */
/*
 * Copyright (c) 2016 BayLibre, SAS
 * Author: Neil Armstrong <narmstrong@baylibre.com>
 */

#ifndef DT_BINDINGS_RESET_AMLOGIC_MESON_G12A_AOCLK
#define DT_BINDINGS_RESET_AMLOGIC_MESON_G12A_AOCLK

#define RESET_AO_IR_IN		0
#define RESET_AO_UART		1
#define RESET_AO_I2C_M		2
#define RESET_AO_I2C_S		3
#define RESET_AO_SAR_ADC	4
#define RESET_AO_UART2		5
#define RESET_AO_IR_OUT		6

#endif
